﻿// Check for flipped images
// Keith Gilbert
// www.gilbertconsulting.com
// 2012-11-06

#target indesign

// The script will not report the following cases properly
//     A graphic that has the frame flipped both horizontally and vertically (since this is the same as a 180 degree rotation) will not be reported
//     A graphic that has the image flipped both horizontally and vertically (since this is the same as a 180 degree rotation) will not be reported
//     A graphic that has both the frame and the image flipped the same way, will not be reported, since the image is flipped back and the appearance is not changed.
//     Pasted graphics that are inline or anchored are not reported.

// Check to see whether any InDesign documents are open.
// If no documents are open, display an error message.
if (app.documents.length > 0) {
	var myDoc = app.activeDocument;
	// Turn off screen redraw while the script runs
	var userRedrawSetting = app.scriptPreferences.enableRedraw;
	app.scriptPreferences.enableRedraw = false;
	// Set the measurement system to points and the origin to page
	userHoriz = myDoc.viewPreferences.horizontalMeasurementUnits;
	userVert = myDoc.viewPreferences.verticalMeasurementUnits;
	userOrigin = myDoc.viewPreferences.rulerOrigin;
	myDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
	myDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
	myDoc.viewPreferences.rulerOrigin = RulerOrigin.PAGE_ORIGIN;
	myDoc.zeroPoint = [0,0];
	
	var myFlippedGraphicsArray = new Array;
	// Iterate through all the linked images. This will find all the images, even in anchored/inlines and tables
	var links = myDoc.links;
	for (var i = links.length-1; i >= 0; i--) {
		var myItem = links[i];
		if  (((myItem.parent.absoluteFlip == Flip.NONE) && (myItem.parent.parent.flip == Flip.NONE)) || ((myItem.parent.absoluteFlip == Flip.HORIZONTAL) && (myItem.parent.parent.flip == Flip.HORIZONTAL)) || ((myItem.parent.absoluteFlip == Flip.VERTICAL) && (myItem.parent.parent.flip == Flip.VERTICAL))) {
			// the object isn't flipped, or is flipped and then flipped back
		}
		else { // the object or the container is flipped
			// store the name of the graphic, the page number, the flip value of the image, and the flip value of the frame to the array
			myFlippedGraphicsArray.push(myItem.name,myItem.parent.parent.parentPage.name,myFlipLookUp(myItem.parent.absoluteFlip),myFlipLookUp(myItem.parent.parent.absoluteFlip));
		}
	}
	// Iterate through all the objects. This will find all the pasted graphics, except anchored/inline and tables
	var myObjectArray = myDoc.pageItems.everyItem().getElements();
	for (var j = 0; j < myObjectArray.length; j++) { // for every object in the file...
		var myItem = myObjectArray[j];
		switch(myItem.constructor.name){
			case "Table":
			case "Cell":
			case "Character":
			case "Word":
			case "TextStyleRange":
			case "Line":
			case "Paragraph":
			case "TextColumn":
			case "Text":
				break;
			case "Rectangle":
			case "Oval":
			case "Polygon":
			case "GraphicLine":
			case "Image":
			case "PDF":
			case "EPS":
				if (myItem.graphics[0] != null) {
					if  (((myItem.graphics[0].absoluteFlip == Flip.NONE) && (myItem.absoluteFlip == Flip.NONE)) || ((myItem.graphics[0].absoluteFlip == Flip.HORIZONTAL) && (myItem.absoluteFlip == Flip.HORIZONTAL)) || ((myItem.graphics[0].absoluteFlip == Flip.VERTICAL) && (myItem.absoluteFlip == Flip.VERTICAL))) {
						// the object isn't flipped, or is flipped and then flipped back
					}
					else { // the object or the container is flipped
						if (myItem.graphics[0].itemLink != null) {
							// We already located these by iterating through the links, so ignore
						}
						else {
							myFlippedGraphicsArray.push("A pasted graphic",myItem.parentPage.name,myFlipLookUp(myItem.graphics[0].absoluteFlip),myFlipLookUp(myItem.absoluteFlip));
						}
					}
				}
			default:
				break;
		}		
	}
	// Report on any flipped graphics found
	var myResult = "";
	if (myFlippedGraphicsArray.length > 0) {
		for (var i = 0; i < myFlippedGraphicsArray.length; i++) {
			myResult = myResult+myFlippedGraphicsArray[i] +" on page "+myFlippedGraphicsArray[i+1]+" is flipped. Image: "+myFlippedGraphicsArray[i+2]+", Frame: "+myFlippedGraphicsArray[i+3]+"\r";
			i = i+3;
		}
		var myResultDoc = app.documents.add();
		myResultDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
		myResultDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
		myResultDoc.viewPreferences.rulerOrigin = RulerOrigin.PAGE_ORIGIN;
		myResultDoc.zeroPoint = [0,0];
		var myPage = myResultDoc.pages.item(0);
		//Get page width and page height using the function "myGetBounds".
		var myBounds = myGetBounds(myResultDoc, myPage);
		var myTextFrame = myPage.textFrames.add();
		//Resize the text frame to match the publication margins.
		myTextFrame.geometricBounds = myBounds;
		myTextFrame.contents = myResult;
		myTextFrame.contents.justification = Justification.leftAlign;
	}
	else {
		alert("This document contains no flipped images.");
	}


	// Return the measurement system and origin to the way it was
	myDoc.viewPreferences.horizontalMeasurementUnits = userHoriz;
	myDoc.viewPreferences.verticalMeasurementUnits = userVert;
	myDoc.viewPreferences.rulerOrigin = userOrigin;
	// Turn screen redraw back to what it was
	app.scriptPreferences.enableRedraw = userRedrawSetting;
}
else {
	// No documents are open, so display an error message.
	alert("No InDesign documents are open. Please open a document and try again.")
}

function myFlipLookUp (myValue) {
	switch(myValue) {
		case "1852796517":
			return "None";
			break;
		case "1752134266":
			return "Horizontal";
			break;
		case "1986359924":
			return "Vertical";
			break;
		case "1215977068": // These two aren't actually read. An object that is 
		case "1651471464":
			return "Horizontal and Vertical";
			break;
		default:
			return "unresolved";
			break;
	}
}


function myGetBounds(myDocument, myPage){
	var myPageWidth = myDocument.documentPreferences.pageWidth;
	var myPageHeight = myDocument.documentPreferences.pageHeight
	if(myPage.side == PageSideOptions.leftHand){
		var myX2 = myPage.marginPreferences.left;
		var myX1 = myPage.marginPreferences.right;
	}
	else{
		var myX1 = myPage.marginPreferences.left;
		var myX2 = myPage.marginPreferences.right;
	}
	var myY1 = myPage.marginPreferences.top;
	var myX2 = myPageWidth - myX2;
	var myY2 = myPageHeight - myPage.marginPreferences.bottom;
	return [myY1, myX1, myY2, myX2];
}
